Class: URBANopt::GeoJSON::DistrictSystem
- Defined in:
- lib/urbanopt/geojson/district_system.rb
Instance Attribute Summary
Attributes inherited from Feature
Instance Method Summary collapse
-
#feature_type ⇒ Object
Used to describe the feature type using the base method from the Feature class.
-
#schema_file ⇒ Object
Returns the district system properties schema.
Methods inherited from Feature
#calculate_aspect_ratio, #create_origin_lat_lon, #find_feature_center, #get_min_lon_lat, #get_multi_polygons, #get_perimeter_multiplier, #id, #initialize, #method_missing, #name, #schema
Constructor Details
This class inherits a constructor from URBANopt::GeoJSON::Feature
Dynamic Method Handling
This class handles dynamic methods through the method_missing method in the class URBANopt::GeoJSON::Feature
Instance Method Details
#feature_type ⇒ Object
Used to describe the feature type using the base method from the Feature class.
13 14 15 |
# File 'lib/urbanopt/geojson/district_system.rb', line 13 def feature_type 'District System' end |
#schema_file ⇒ Object
Returns the district system properties schema.
19 20 21 |
# File 'lib/urbanopt/geojson/district_system.rb', line 19 def schema_file return File.join(File.dirname(__FILE__), 'schema', 'district_system_properties.json') end |