Class: URBANopt::GeoJSON::Region
- Defined in:
- lib/urbanopt/geojson/region.rb
Instance Attribute Summary
Attributes inherited from Feature
Instance Method Summary collapse
-
#feature_type ⇒ Object
Used to describe the Region feature type using the base method from the Feature class.
-
#schema_file ⇒ Object
Returns the region_properties schema.
Methods inherited from Feature
#calculate_aspect_ratio, #create_origin_lat_lon, #find_feature_center, #get_min_lon_lat, #get_multi_polygons, #get_perimeter_multiplier, #id, #initialize, #method_missing, #name, #schema
Constructor Details
This class inherits a constructor from URBANopt::GeoJSON::Feature
Dynamic Method Handling
This class handles dynamic methods through the method_missing method in the class URBANopt::GeoJSON::Feature
Instance Method Details
#feature_type ⇒ Object
Used to describe the Region feature type using the base method from the Feature class.
16 17 18 |
# File 'lib/urbanopt/geojson/region.rb', line 16 def feature_type 'Region' end |
#schema_file ⇒ Object
Returns the region_properties schema.
22 23 24 |
# File 'lib/urbanopt/geojson/region.rb', line 22 def schema_file return File.join(File.dirname(__FILE__), 'schema', 'region_properties.json') end |